@@ -1,9 +1,12 @@
 from .sdk import *
-__version__ = '1.1.22'
 __all__ =[
     'TGAnalytics',
     'LoggingConsumer',
     'BatchConsumer',
     'AsyncBatchConsumer',
-    '__version__'
+    'TGAException',
+    'TGAIllegalDataException',
+    'TGANetworkException',
+    'ROTATE_MODE',
+    'DebugConsumer',
 ]
\ No newline at end of file
